///////////////////////////////////////////////////////////////////////////////
// $Workfile: WWhizInterfaceLoader.cpp $
// $Archive: /WorkspaceWhiz/Src/Shared/WWhizInterfaceLoader.cpp $
// $Date: 2000/10/04 02:32:58 $ $Revision: 1.2 $ $Author: osolo $
///////////////////////////////////////////////////////////////////////////////
// This source file is part of the Workspace Whiz! source distribution and
// is Copyright 1997-2000 by Joshua C. Jensen.  (http://www.workspacewhiz.com/)
//
// The code presented in this file may be freely used and modified for all
// non-commercial purposes so long as due credit is given and this header is
// left intact.
//
// Permission for commercial usage is obtained by contacting the author, Joshua
// C. Jensen, at jjensen@workspacewhiz.com.
///////////////////////////////////////////////////////////////////////////////
#include <afx.h>
#include "WWhizInterface.h"
#include "WWhizInterfaceLoader.h"

#define USE_WWHIZ_AICLOADER 0
#if USE_WWHIZ_AICLOADER
#include "AICLoader.h"
#include "AddInComm.h"
#endif //USE_WWHIZ_AICLOADER

class WWhizInterface;

typedef WWhizInterface* (*pfnWWhizCreate)(IApplication* pApplication);
typedef void (*pfnWWhizDestroy)();

static HINSTANCE s_wwhizInst;
static pfnWWhizCreate s_wwhizCreate;
static pfnWWhizDestroy s_wwhizDestroy;
static WWhizInterface* s_wwhizInterface;

// this object will automatically clean up after the loader by calling
// WWhizDestroy() when the program exists.
static struct auto_cleanup { ~auto_cleanup() { WWhizInterfaceDestroy(); } } autoc;

static void GetWWhizDirectory(LPTSTR wwhizPath)
{
    HKEY hKey;
    DWORD type;
	DWORD pathLen = _MAX_PATH;

	wwhizPath[0] = 0;
    if (::RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\Workspace Whiz!",
			0, KEY_READ, &hKey) == ERROR_SUCCESS)
    {
        if (::RegQueryValueEx(hKey, "Path", 0, &type, 
            (LPBYTE)wwhizPath, &pathLen) == ERROR_SUCCESS)
        {
        }

        ::RegCloseKey(hKey);
    }
}


WWhizInterface* __cdecl WWhizInterfaceCreate(HINSTANCE hInstance, IApplication* pApplication)
{
	// Only allow one interface per instance.
	if (s_wwhizInterface)
		return s_wwhizInterface;

	TCHAR wwhizPath[_MAX_PATH];
	GetWWhizDirectory(wwhizPath);
	if (wwhizPath[_tcslen(wwhizPath) - 1] != '\\')
		_tcscat(wwhizPath, "\\");

#ifdef _DEBUG
	LPCTSTR filename = "WWhizInterface2D.mod";
#else
	LPCTSTR filename = "WWhizInterface2.mod";
#endif //_DEBUG

	// Get the module name and strip the module filename from it, leaving the
	// module path.
	TCHAR moduleName[_MAX_PATH];
	moduleName[0] = 0;
	::GetModuleFileName(hInstance, (TCHAR*)&moduleName, _MAX_PATH);
	TCHAR* ptr = _tcsrchr(moduleName, '\\');
	if (ptr)
	{
		ptr++;
		*ptr = 0;
	}

	_tcscat(moduleName, _T(filename));
	s_wwhizInst = LoadLibrary(moduleName);
	if (!s_wwhizInst)
	{
		_tcscat(wwhizPath, filename);
		s_wwhizInst = LoadLibrary(wwhizPath);
		if (!s_wwhizInst)
		{
				return NULL;
		}
	}

	s_wwhizCreate = (pfnWWhizCreate)GetProcAddress(s_wwhizInst, "WWhizInterfaceCreate");
	s_wwhizDestroy = (pfnWWhizDestroy)GetProcAddress(s_wwhizInst, "WWhizInterfaceDestroy");
	if (!s_wwhizCreate)
	{
		WWhizInterfaceDestroy();
		return NULL;
	}

	s_wwhizInterface = (*s_wwhizCreate)(pApplication);

	return s_wwhizInterface;
}


void __cdecl WWhizInterfaceDestroy()
{
	if (s_wwhizInst)
	{
		if (s_wwhizDestroy)
		{
			(*s_wwhizDestroy)();
		}
		BOOL ret = ::FreeLibrary(s_wwhizInst);
		s_wwhizInst = NULL;
	}
	s_wwhizCreate = NULL;
	s_wwhizInterface = NULL;
}
